# Microeconometrics
# CLASS 8
# Treatment effects
# Wiktor Budzinski, Marek Giergiczny

#rm(packages)

# Loading libraries
library(psych)
library(aod)
library(AER)
library(rdrobust)
library(plm)
library(stargazer)
library(endogenous)
library(sampleSelection)
library(readxl)
library(GJRM)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 8")  

############################################ Exercise 1 ############################################
# Insurance and medicine expenses

rm(list=ls())
# ctrl+l

############################################
# 1.1 Use data from me.medexp3.rds

medexp3 <- readRDS("me.medexp3.rds")

describe(medexp3)

############################################
# Check if the logarithm of expenses for medicines depends on having additional insurance from the employer

# Compare OLS (Ordinary Least Squares) model and 2SLS (2 Stage Least Squares) model

# Logarithm of drug expenditure explained by insurance and additional control variables
Eq1 <- ldrugexp ~ hi_empunion + totchr + age + female + blhisp + linc
model1 <- lm(Eq1, data=medexp3)
summary(model1)

# We use 2SLS to (additionally) control the endogenous situation
# As an instrumental variable, we use 'ssiratio' (which tells us how much of the respondent's income comes from allowances (~benefits))
Eq2 <- ldrugexp ~ hi_empunion + totchr + age + female + blhisp + linc | ssiratio + totchr + age + female + blhisp + linc
model2 <- ivreg(Eq2, data=medexp3)
summary(model2)
# Controlling endogeneity changes the sign of the coef

# Endogenous treatment model
Eq3a <- medexp3$ldrugexp ~ medexp3$totchr + medexp3$age + medexp3$female + medexp3$blhisp + medexp3$linc 
Eq3b <- medexp3$hi_empunion ~ medexp3$ssiratio + medexp3$totchr + medexp3$age + medexp3$female + medexp3$blhisp + medexp3$linc
model3<-hybrid(Eq3a, probit = Eq3b)
summary(model3)

# Alternatively: 
Eq3c <- hi_empunion ~ ssiratio + totchr + age + female + blhisp + linc
model3b<-treatReg(Eq3c, Eq1, data = medexp3)
summary(model3b)

# Treatment specific effects 
model4 <- hybrid(Eq3a, probit = Eq3b, modifiers = medexp3$hi_empunion ~ medexp3$linc)
summary(model4)

# Switching regression
  # Outcome 2 is when selection = 1
Eq4 <- ldrugexp ~ totchr + age + female + blhisp + linc
model5<-selection(Eq3c, list(Eq4, Eq4), data = medexp3, type = 5) # Tobit - 5
summary(model5)


############################################ Exercise 2 ############################################
# Treatment effect with binary outcome
rm(list=ls())

data("meps", package = "GJRM")
#meps$income <- meps$income/100000 
meps$linc <- log(meps$income+1)
meps$age <- meps$age/10
meps$education <- meps$education/10
meps$vis <- meps$visits.hosp == 1


treat <- private ~ linc + education  + age + as.factor(race) + as.factor(limitation) + gender + hypertension +hyperlipidemia + diabetes
m1 <- glm(treat, family=binomial(link="probit"), data = meps)
summary(m1)
outcome <-  vis ~ private + education + age + as.factor(race) + as.factor(limitation) + gender + hypertension +hyperlipidemia + diabetes
m2<-glm(outcome, family=binomial(link="probit"), data = meps)
summary(m2)

# Does not work
m3<-treatReg(treat, outcome, data = meps)

f.list <- list(treat, outcome)
mr <- c("probit", "probit")
m3 <- gjrm(f.list, data = meps, model = "B",
               margins = mr)
conv.check(m3)
summary(m3)

############################################ Exercise 3 ############################################
# Senate elections

rm(list=ls())

senate <- readRDS("me.senate.rds")

describe(senate)
str(senate)

# Margin is defines as:
#   Democratic vote share minus the vote share of the strongest opponent and ranging from -100 to 100 percentage points.
# Check how the victory of a democratic party in previous elections to the senate influenses their result in subsequent elections
# The discountinuity is at 0 (i.e. Margin>0 means winning in previous elections)

hist(senate$margin)
# Local-Polynomial RD Estimation with Robust Confidence Intervals (~ regression with discontinuity)
model1 <- rdrobust(senate$vote, senate$margin)
summary(model1)

# It turns out that winning the previous election increases the result in the next election by 7.41 percentage points
# The algorithm automatically selects which observations we include in the regression (BW est. (h) = 17.754)
# I.e. we include observations for which the result in previous elections was from -17.754 to 17.754


# Model with additional estimates with corrections
# "If all is specified, rdrobust reports three different procedures:
#   (i)     conventional RD estimates with conventional standard errors.
#   (ii)    bias-corrected estimates with conventional standard errors.
#   (iii)   bias-corrected estimates with robust standard errors."

model2 <- rdrobust(senate$vote, senate$margin, all=senate$margin)
summary(model2)
# Robust differs in standard error estimation

# We can change 'h' manually
# "h specifies the main bandwidth used to construct the RD point estimator.
# If not specified, bandwidth h is computed by the companion command rdbwselect.
# If two bandwidths are specified, the first bandwidth is used for the data below the cutoff
# and the second bandwidth is used for the data above the cutoff."
model3 <- rdrobust(senate$vote, senate$margin, h=15)
summary(model3)


# We can also change 'p'
# "p specifies the order of the local-polynomial used to construct the point-estimator;
# default is p = 1 (local linear regression)."
model4<- rdrobust(senate$vote, senate$margin, p=4)
summary(model4)

# By default first degree polynomial is used, but the higher polynomial level, the more observations needed (h increases)

# Graph of regression with discontinuity
rdplot(senate$vote, senate$margin)

# Graph with parameters that command rdrobust estimated
rdplot(senate$vote, senate$margin, h=17.754, p=1)

# Closeup
rdplot(senate$vote, senate$margin, h=17.754, p=1, ci=TRUE)                                             # "ci - optional graphical option to display confidence intervals of selected level for each bin"
rdplot(senate$vote, senate$margin, h=17.754, p=1, ci=TRUE, shade=TRUE)                                 # "shade - optional graphical option to replace confidence intervals with shaded areas"
rdplot(senate$vote, senate$margin, h=17.754, p=1, ci=TRUE, shade=TRUE, subset=abs(senate$margin)<20)   # "subset - an optional vector specifying a subset of observations to be used"

# Notice that in fact the data does not look so "nice"
senate_subset <- subset(senate, abs(margin)<20)
plot(senate_subset$margin, senate_subset$vote)

############################################ Exercise 4 #########################################
# Fuzzy RD as IV

rm(list=ls())
Class <- read_excel("Class_size.xls")

Eq1 <- avgverb ~ classize + cohsize + tipuach
m1 <- lm(Eq1, data = Class)
summary(m1)

Class$func <-  Class$cohsize/(floor((Class$cohsize-1)/40)+1)

summary(Class$func)
scatter.smooth(Class$classize ~ Class$cohsize, col="grey",
               las=1, ylab="Class size", xlab="Cohort size")
points(Class$cohsize, Class$func, col="red")

Eq2 <- avgverb ~ classize + cohsize + tipuach | func + cohsize + tipuach
m2 <- ivreg(Eq2, data=Class)
summary(m2, diagnostics=TRUE) # Endogeneity is significant

# Similarly for math
Eq3 <- avgmath ~ classize + cohsize + tipuach 
m3 <- lm(Eq3, data=Class)
summary(m3)

Eq4 <- avgmath ~ classize + cohsize + tipuach | func + cohsize + tipuach
m4 <- ivreg(Eq4, data=Class)
summary(m4, diagnostics=TRUE)
