library(psych)
library(sampleSelection)
library(stargazer)
library(censReg)
library(mvtnorm)
library(endogenous)
library(AER)

library(rdrobust)


#####################  Simulation example no. 1 ########################
# Endogenous treatment
  # Checking whether the "hybrid" function actually changes the sign

rm(list=ls())
set.seed(1234567) # setting seed

x1 <- rnorm(2000,0,2)
x2 <- rnorm(2000,0,1)
z1 <- runif(2000,0,1)
sigma <- matrix(c(1,0.5,0.5,1), ncol=2)
eps <- rmvnorm(n=2000, mean=c(0,0), sigma=sigma)

y1star <- -1 - 0.5*x1 + z1 + eps[,1]
treat <- y1star > 0
summary(treat)

y2<- -1 + treat+x1 + 0.5*x2 +  eps[,2]

summary(lm(y2 ~ treat + x1 + x2))

model3<-hybrid(y2 ~ x1 + x2, probit = treat ~ z1 + x1)
summary(model3)

########################## Simulation example no. 2 ##########################
# RDD vs. nonlinearity

rm(list=ls())

set.seed(1234567) # setting seed
eps <- rnorm(500,0,1)
margin <- rnorm(500,0,1)
d <- (margin > 0) # treatment
nlm <- 1/(1 + exp(-6*margin))
plot(margin, nlm)

y <- 2 + 3*nlm + eps
plot(margin, y)

model1<- rdrobust(y, margin)
summary(model1)

model1b<- rdrobust(y, margin, h=1, p = 1)
summary(model1b)

model1c<- rdrobust(y, margin, h=2, p = 1)
summary(model1c)

model1d<- rdrobust(y, margin, h=1, p = 2)
summary(model1d)

model1e<- rdrobust(y, margin, h=2, p = 3)
summary(model1e)


y2 <- 2 + 3*nlm + 0.8*d + eps
model2 <- rdrobust(y2, margin)
summary(model2)

########################## Simulation example no. 3 ##########################
# Fuzzy RDD 

rm(list=ls())

set.seed(1234567) # setting seed
sigma <- matrix(c(1,0.5,0.5,1), ncol=2)
eps <- rmvnorm(n=2000, mean=c(0,0), sigma=sigma)
margin <- rnorm(2000,0,1)
d <- 2*(margin > 0) + margin + eps[,1]
#ystar <- 2*(margin > 0) + eps[,1]
plot(margin, d)

y <- margin + 0.2*margin^2 +2*d + eps[,2]

model1<- rdrobust(y, margin, fuzzy = d)
summary(model1)

t<- (margin > 0)
model2 <- ivreg(y ~ margin + I(margin^2) + d | margin + I(margin^2) + t)
summary(model2)

# for comparison
summary(lm(y ~ margin + I(margin^2) + d))